package es.rickyepoderi.keycloaksample;

import android.util.Log;

import java.io.DataOutputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;

public class RefreshTask extends BaseTask<TokenResponse> {

    private static final String TAG = "RefreshTask";

    public RefreshTask(MainActivity activity, Callback<TokenResponse> callback) {
        super(activity, callback);
    }

    @Override
    protected TokenResponse doInBackground(String... strings) {
        HttpURLConnection conn = null;
        try {
            String refreshToken = strings[0];
            URL keycloak = activity.createTokenURL();
            String urlParameters = activity.createRefreshPostData(refreshToken);
            byte[] postData = urlParameters.getBytes(StandardCharsets.UTF_8);
            conn = (HttpURLConnection) keycloak.openConnection();
            conn.setDoOutput(true);
            conn.setInstanceFollowRedirects(false);
            conn.setRequestMethod("POST");
            conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            conn.setRequestProperty("charset", "utf-8");
            // if confidential you need the basic auth
            //conn.setRequestProperty("Authorization", "Basic " +
            //        Base64.encodeToString("KeycloakSample:516866c3-de8b-4d74-8856-92d604bedad3".getBytes(), Base64.DEFAULT));
            conn.setRequestProperty("Content-Length", Integer.toString(postData.length));
            conn.setUseCaches(false);
            try (DataOutputStream wr = new DataOutputStream(conn.getOutputStream())) {
                wr.write(postData);
            }
            if (conn.getResponseCode() == 200) {
                return new TokenResponse(conn.getInputStream(), activity.getSignatureKey());
            } else {
                return new TokenResponse(conn.getResponseCode() + ": " + conn.getResponseMessage());
            }
        } catch (IOException|InvalidKeySpecException|NoSuchAlgorithmException e) {
            Log.d(TAG, "Error calling keycloak", e);
            return new TokenResponse(e.getMessage());
        } finally {
            if (conn != null) {
                conn.disconnect();
            }
        }
    }
}
